/*
 * ServerElement.cs
 *
 * Testing Framework
 * Version: 1.0
 *
 * Copyright (c) 2006, TopCoder, Inc. All rights reserved
 *
 * Authors: real_vg, TCSDEVELOPER
 */

using System;
using NAnt.Core;
using NAnt.Core.Attributes;
using NAnt.Core.Types;
using TopCoder.Test.Framework.Servers;

namespace TopCoder.Test.Framework
{
    /// <summary>
    /// Represents "server" XML element to be used in the NAnt build file.
    /// This element should be used inside the "serverapptest" NAnt task
    /// (which is represented by the <see cref="ServerAppTestTask"/> class.
    /// </summary>
    ///
    /// <remarks>
    /// The attributes supported by this element are:
    /// <list type="table">
    /// <listheader>
    ///     <term>Attribute name</term><description>Description</description>
    /// </listheader>
    /// <item>
    ///     <term><b>class</b></term>
    ///     <description>
    ///     The name of class representing the server. It should be a valid name of a
    ///     class derived from the <see cref="Server"/> abstract class. This attribute is required.
    ///     </description>
    /// </item>
    /// <item>
    ///     <term><b>startserver</b></term>
    ///     <description>
    ///     Specifies if the server should be started by the component
    ///     when running tests. This attribute is not required, by default is true.
    ///     </description>
    /// </item>
    /// <item>
    ///     <term><b>initserver</b></term>
    ///     <description>
    ///     Specifies if the server should be initialized with data by the component
    ///     when running tests. This attribute is not required, by default is true.
    ///     </description>
    /// </item>
    /// </list>
    /// The nested XML elements supported are:
    /// <list type="table">
    /// <listheader>
    ///     <term>Element name</term><description>Description</description>
    /// </listheader>
    /// <item>
    ///     <term><b>options/option</b></term>
    ///     <description>
    ///     Can be used zero or more times, representing a particular option of the server.
    ///     </description>
    /// </item>
    /// <item>
    ///     <term><b>credentials</b></term>
    ///     <description>
    ///     Can be used zero or one times, representing a credential used to login to the server.
    ///     </description>
    /// </item>
    /// <item>
    ///     <term><b>sourcefiles</b></term>
    ///     <description>
    ///     Can be used zero or more times, representing files used to initialize the server with data.
    ///     </description>
    /// </item>
    /// </list>
    /// </remarks>
    ///
    /// <example>
    /// Here's a sample server element: <code><![CDATA[
    /// <server class="TopCoder.Test.Framework.Servers.Web.IISServer" initserver="true" startserver="true">
    ///     <options>
    ///         <!-- The name of server service -->
    ///         <option name="service_name" value = "w3svc"/>
    ///         <!-- The directory to deploy application to-->
    ///         <option name="deployment_dir" value = "${deployment_dir}"/>
    ///         <!-- The timeout is set to one minute in milliseconds-->
    ///         <option name="timeout" value = "60000"/>
    ///     </options>
    ///
    ///     <sourcefiles basedir="deploysource">
    ///         <!-- This should be a fileset specifying the files to deploy-->
    ///         <includes name="**"/>
    ///     </sourcefiles>
    /// </server>
    /// ]]></code>
    /// </example>
    ///
    /// <threadsafety static="false" instance="false"/>
    [ElementName("server")]
    public class ServerElement : Element
    {
        /// <summary>
        /// This variable represents the "class" attribute.
        /// "class" attribute represents the name of class derived from Server,
        /// which represents the server described by this element.
        /// This attribute is required. Initial value is null.
        /// It can be accessed via "ClassName" property.
        /// </summary>
        private string className;

        /// <summary>
        /// This variable represents the server described by this element
        /// Initially is null. Lazily initialized in GetServerInstance().
        /// </summary>
        private Server server;

        /// <summary>
        /// This variable represents the "startserver" attribute.
        /// "startserver" attribute specifies if the server should
        /// be started by the component when running tests.
        /// This attribute is not required, the default value is true.
        /// Initially is set to true.
        /// Can be accessed via the StartServer property.
        /// </summary>
        private bool startServer = true;

        /// <summary>
        /// This variable represents the "initserver" attribute.
        /// "initserver" attribute specifies if the server should
        /// be initialized with data by the component when running tests.
        /// This attribute is not required, the default value is true.
        /// Initially is set to true.
        /// Can be accessed via the InitServer property.
        /// </summary>
        private bool initServer = true;

        /// <summary>
        /// This variable represents the collection of nested "option" XML elements, which
        /// represent the options of the server.
        /// The options can be any key-value pairs, where keys and values are strings.
        /// Class derived from Server, as specified by the "class" attribute,
        /// may interpret the options freely.
        /// This nested element can possibly be omitted if the specified server
        /// doesn't require it.
        /// Can be accessed via the "Options" property.
        /// </summary>
        private OptionCollection options = new OptionCollection();

        /// <summary>
        /// This variable represents the nested "credentials" XML element, which
        /// represents the credentials used to authenticate to the server.
        /// This includes user name, password an domain.
        /// This nested element can possibly be omitted,
        /// it depends on the particular server.
        /// Can be accessed via the "Credentials" property.
        /// </summary>
        private Credential credentials = new Credential();

        /// <summary>
        /// This variable represents the nested "sourcefiles" XML element, which
        /// represents the files used when initializing the server with data.
        /// This can be an application to deploy for an application server; or
        /// some SQL scripts, DB tables, etc, for a data server; or some other files
        /// the particular server may need to be initialized.
        /// This nested element can possibly be omitted if the class derived from Server,
        /// specified by "class" attribute, permits it.
        /// Can be accessed via the "sourceFiles" property.
        /// </summary>
        private FileSet sourceFiles = new FileSet();

        /// <summary>
        /// Gets or sets the name of the class (derived from <see cref="Server"/>)
        /// which represents the server described by this element.
        /// </summary>
        ///
        /// <value>
        /// The name of class (derived from <see cref="Server"/>)
        /// which represents the server described by this element.
        /// This attribute is required.
        /// </value>
        [TaskAttribute("class", Required = true)]
        public string ClassName
        {
            get
            {
                return className;
            }
            set
            {
                className = value;
            }
        }

        /// <summary>
        /// Gets or sets a boolean value that indicates whether
        /// the server should be started by the component when running tests.
        /// </summary>
        ///
        /// <value>
        /// true if the server should be started by the component when running tests;
        /// otherwise, false. This attribute is not required and the default value is true.
        /// </value>
        [TaskAttribute("startserver")]
        public bool StartServer
        {
            get
            {
                return startServer;
            }
            set
            {
                startServer = value;
            }
        }

        /// <summary>
        /// Gets or sets a boolean value that indicates whether
        /// the server should be initialized with data by the component when running tests.
        /// </summary>
        ///
        /// <value>
        /// true if the server should be initialized with data by the component when running tests;
        /// otherwise, false. This attribute is not required and the default value is true.
        /// </value>
        [TaskAttribute("initserver")]
        public bool InitServer
        {
            get
            {
                return initServer;
            }
            set
            {
                initServer = value;
            }
        }

        /// <summary>
        /// Gets or sets the collection of nested "options/option" XML elements, which
        /// represents the options of the server.
        /// </summary>
        ///
        /// <value>
        /// An OptionCollection which represents the options of the server.
        /// It can be any key-value pairs, where key and values are strings.
        /// It could be omitted.
        /// </value>
        [BuildElementCollection("options", "option")]
        public OptionCollection Options
        {
            get
            {
                return options;
            }
        }

        /// <summary>
        /// Gets or sets the "credentials" XML element, which
        /// represents the credentials used to authenticate to the server.
        /// </summary>
        ///
        /// <value>
        /// A Credential which represents the credentials used to authenticate to the server.
        /// It could be omitted.
        /// </value>
        [BuildElement("credentials")]
        public Credential Credentials
        {
            get
            {
                return credentials;
            }
            set
            {
                credentials = value;
            }
        }

        /// <summary>
        /// Gets or sets the "sourcefiles" XML element, which
        /// represents the files used when initializing the server with data.
        /// </summary>
        ///
        /// <remarks>
        /// This can be an application to deploy for an application server; or
        /// some SQL scripts, DB tables, etc, for a data server; or some other files
        /// the particular server may need to be initialized.
        /// </remarks>
        ///
        /// <value>
        /// A FileSet which represents the files used when initializing the server with data.
        /// It could be omitted.
        /// </value>
        [BuildElement("sourcefiles")]
        public FileSet SourceFiles
        {
            get
            {
                return sourceFiles;
            }
            set
            {
                sourceFiles = value;
            }
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="ServerElement"/> class.
        /// </summary>
        public ServerElement()
        {
            // do nothing
        }

        /// <summary>
        /// Gets an instance of class derived from <see cref="Server"/> class,
        /// which represents the server described by this XML element.
        /// The name of class to create instance of is taken from the <see cref="ClassName"/> property.
        /// </summary>
        ///
        /// <returns>
        /// An instance of class derived from <see cref="Server"/> class which
        /// represents the server described by this XML element.
        /// </returns>
        ///
        /// <exception cref="ServerException">
        /// Any error happens when creating an instance of the server.
        /// </exception>
        public Server GetServerInstance()
        {
            // if we have instantiate it previously, return it
            if (server != null)
            {
                return server;
            }

            try
            {
                // create an instance (must be of Server type)
                Type type = Type.GetType(ClassName, true);
                object instance = Activator.CreateInstance(type);
                if (!(instance is Server))
                {
                    throw new ServerException(string.Format(
                        "'{0}' is not of type Server", ClassName));
                }

                // assign properties
                server = (Server) instance;
                server.StartServer = StartServer;
                server.InitServer = InitServer;
                server.Options = Options;
                server.Credentials = Credentials;
                server.SourceFiles = SourceFiles;
                server.Project = Project;

                return server;
            }
            catch (ServerException)
            {
                throw;
            }
            catch (Exception e)
            {
                throw new ServerException("Error creating server instance", e);
            }
        }
    }
}
