/*
 * DatabaseServer.cs
 *
 * Testing Framework
 * Version: 1.0
 *
 * Copyright (c) 2006, TopCoder, Inc. All rights reserved
 *
 * Authors: real_vg, TCSDEVELOPER
 */

namespace TopCoder.Test.Framework.Servers
{
    /// <summary>
    /// Provides an abstract base class for all the classes representing
    /// a database server to be used when running tests.
    /// </summary>
    ///
    /// <remarks>
    /// Note that no concrete database server class are provided by this component.
    /// This class extends <see cref="Server"/> by adding the following options:
    /// <list type="bullet">
    /// <item><description>
    /// <see cref="DatabaseDriver"/> - the name of the database driver to use for connecting to the database.
    /// </description></item>
    /// <item><description>
    /// <see cref="DatabaseConnectionString"/> - the database connection string to use
    /// for connecting to the database.
    /// </description></item>
    /// <item><description>
    /// <see cref="SqlInitScript"/> - the name of file, containing a SQL script used to initialize the data.
    /// </description></item>
    /// <item><description>
    /// <see cref="SqlCleanUpScript"/> - the name of file, containing a SQL script used to clean-up the data.
    /// </description></item>
    /// </list>
    /// </remarks>
    ///
    /// <threadsafety static="false" instance="false"/>
    public abstract class DatabaseServer : Server
    {
        /// <summary>
        /// A string constant that represents the name (key) of the option, used by derived classes,
        /// for getting the name of the database driver to use for connecting to the database.
        /// </summary>
        ///
        /// <remarks>
        /// Note that derived classes can treat options in any way they like to do.
        /// </remarks>
        public static readonly string DB_DRIVER = "db_driver";

        /// <summary>
        /// A string constant that represents the name (key) of the option, used by derived classes,
        /// for getting the database connection string to use for connecting to the database.
        /// </summary>
        ///
        /// <remarks>
        /// Note that derived classes can treat options in any way they like to do.
        /// </remarks>
        public static readonly string DB_CONNECTION_STR = "db_connection_str";

        /// <summary>
        /// A string constant that represents the name (key) of the option, used by derived classes,
        /// for getting the name of file, containing a SQL script used to initialize the data.
        /// </summary>
        ///
        /// <remarks>
        /// Note that derived classes can treat options in any way they like to do.
        /// The most common way of use of this options will be
        /// executing the specified script in the InitData() method.
        /// </remarks>
        public static readonly string SQL_INIT_SCRIPT = "sql_init_script";

        /// <summary>
        /// A string constant that represents the name (key) of the option, used by derived classes,
        /// for getting the name of file, containing a SQL script used to clean-up the data.
        /// </summary>
        ///
        /// <remarks>
        /// Note that derived classes can treat options in any way they like to do.
        /// The most common way of use of this options will be
        /// executing the specified script in the CleanUpData() method.
        /// </remarks>
        public static readonly string SQL_CLEANUP_SCRIPT = "sql_cleanup_script";

        /// <summary>
        /// Gets the name of the database driver to use for connecting to database.
        /// </summary>
        ///
        /// <value>
        /// The name of the database driver to use for connecting to database.
        /// </value>
        ///
        /// <exception cref="ServerException">
        /// The value doesn't exist.
        /// </exception>
        public string DatabaseDriver
        {
            get
            {
                return OptionHelper.GetString(DB_DRIVER, Options, false);
            }
        }

        /// <summary>
        /// Gets the database connection string to use for connecting to the database.
        /// </summary>
        ///
        /// <value>
        /// The database connection string to use for connecting to the database.
        /// </value>
        ///
        /// <exception cref="ServerException">
        /// The value doesn't exist.
        /// </exception>
        public string DatabaseConnectionString
        {
            get
            {
                return OptionHelper.GetString(DB_CONNECTION_STR, Options, false);
            }
        }

        /// <summary>
        /// Gets the name of file, containing a SQL script used to initialize the data.
        /// </summary>
        ///
        /// <value>
        /// The name of file, containing a SQL script used to initialize the data.
        /// </value>
        ///
        /// <exception cref="ServerException">
        /// The value doesn't exist.
        /// </exception>
        public string SqlInitScript
        {
            get
            {
                return OptionHelper.GetString(SQL_INIT_SCRIPT, Options, false);
            }
        }

        /// <summary>
        /// Gets the name of file, containing a SQL script used to clean-up the data.
        /// </summary>
        ///
        /// <value>
        /// The name of file, containing a SQL script used to clean-up the data.
        /// </value>
        ///
        /// <exception cref="ServerException">
        /// The value doesn't exist.
        /// </exception>
        public string SqlCleanUpScript
        {
            get
            {
                return OptionHelper.GetString(SQL_CLEANUP_SCRIPT, Options, false);
            }
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="DatabaseServer"/> class.
        /// </summary>
        protected DatabaseServer()
        {
            // do nothing
        }
    }
}
