/*
 * MyServer.cs
 *
 * Testing Framework
 * Version: 1.0
 *
 * Copyright (c) 2006, TopCoder, Inc. All rights reserved
 *
 * Author: TCSDEVELOPER
 */

using System;
using System.Threading;

namespace TopCoder.Test.Framework.Servers.Tests
{
    /// <summary>
    /// A concrete class derived from <see cref="Server"/>
    /// so that it can be tested in <see cref="TestServer"/>.
    /// </summary>
    ///
    /// <remarks>
    /// Additionally, this class provides flags, which are tick counts,
    /// indicating the last times the methods are called.
    /// This can be used to assert that servers are started/initialized/cleaned/stopped
    /// in the expected order.
    /// Some Options can also be defined to simulate an exception being thrown
    /// from some of the methods.
    /// </remarks>
    ///
    /// <threadsafety static="false" instance="false"/>
    public class MyServer : Server
    {
        /// <summary>
        /// If this Option key is defined, StartUp will throw exception.
        /// Option value can be anything.
        /// </summary>
        public const string StartUpException = "StartUpException";

        /// <summary>
        /// If this Option key is defined, ShutDown will throw exception.
        /// Option value can be anything.
        /// </summary>
        public const string ShutDownException = "ShutDownException";

        /// <summary>
        /// If this Option key is defined, InitData will throw exception.
        /// Option value can be anything.
        /// </summary>
        public const string InitDataException = "InitDataException";

        /// <summary>
        /// If this Option key is defined, CleanUpData will throw exception.
        /// Option value can be anything.
        /// </summary>
        public const string CleanUpDataException = "CleanUpDataException";

        /// <summary>
        /// Simulation delay in milliseconds.
        /// See DateTime.Now for minimal resolution needed.
        /// </summary>
        private const int DelaySimulation = 100;

        /// <summary>
        /// The ticks when StartUp is called. Retrieved from TickStartUp.
        /// </summary>
        private long tickStartUp = -1;

        /// <summary>
        /// The ticks when ShutDown is called. Retrieved from TickShutDown.
        /// </summary>
        private long tickShutDown = -1;

        /// <summary>
        /// The ticks when InitData is called. Retrieved from TickInitData.
        /// </summary>
        private long tickInitData = -1;

        /// <summary>
        /// The ticks when CleanUpData is called. Retrieved from TickCleanUpData.
        /// </summary>
        private long tickCleanUpData = -1;

        /// <summary>
        /// Gets the tick that StartUp was called.
        /// </summary>
        ///
        /// <value>
        /// The tick that StartUp was called.
        /// </value>
        public long TickStartUp
        {
            get
            {
                return tickStartUp;
            }
        }

        /// <summary>
        /// Gets the tick that ShutDown was called.
        /// </summary>
        ///
        /// <value>
        /// The tick that ShutDown was called.
        /// </value>
        public long TickShutDown
        {
            get
            {
                return tickShutDown;
            }
        }

        /// <summary>
        /// Gets the tick that InitData was called.
        /// </summary>
        ///
        /// <value>
        /// The tick that InitData was called.
        /// </value>
        public long TickInitData
        {
            get
            {
                return tickInitData;
            }
        }

        /// <summary>
        /// Gets the tick that CleanUpData was called.
        /// </summary>
        ///
        /// <value>
        /// The tick that CleanUpData was called.
        /// </value>
        public long TickCleanUpData
        {
            get
            {
                return tickCleanUpData;
            }
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="MyServer"/> class.
        /// </summary>
        public MyServer()
        {
            // do nothing
        }

        /// <summary>
        /// Updates TickStartUp and simulates delay.
        /// </summary>
        ///
        /// <exception cref="ServerException">
        /// <see cref="StartUpException"/> is defined in the options.
        /// </exception>
        public override void StartUp()
        {
            tickStartUp = DateTime.Now.Ticks;
            Thread.Sleep(DelaySimulation);

            if ((Options != null) && (Options[StartUpException] != null))
            {
                throw new ServerException("StartUpException defined");
            }
        }

        /// <summary>
        /// Updates TickShutDown and simulates delay.
        /// </summary>
        ///
        /// <exception cref="ServerException">
        /// <see cref="ShutDownException"/> is defined in the options.
        /// </exception>
        public override void ShutDown()
        {
            tickShutDown = DateTime.Now.Ticks;
            Thread.Sleep(DelaySimulation);

            if ((Options != null) && (Options[ShutDownException] != null))
            {
                throw new ServerException("ShutDownException defined");
            }
        }

        /// <summary>
        /// Updates TickInitData and simulates delay.
        /// </summary>
        ///
        /// <exception cref="ServerException">
        /// <see cref="InitDataException"/> is defined in the options.
        /// </exception>
        public override void InitData()
        {
            tickInitData = DateTime.Now.Ticks;
            Thread.Sleep(DelaySimulation);

            if ((Options != null) && (Options[InitDataException] != null))
            {
                throw new ServerException("InitDataException defined");
            }
        }

        /// <summary>
        /// Updates TickCleanUpData and simulates delay.
        /// </summary>
        ///
        /// <exception cref="ServerException">
        /// <see cref="CleanUpDataException"/> is defined in the options.
        /// </exception>
        public override void CleanUpData()
        {
            tickCleanUpData = DateTime.Now.Ticks;
            Thread.Sleep(DelaySimulation);

            if ((Options != null) && (Options[CleanUpDataException] != null))
            {
                throw new ServerException("CleanUpDataException defined");
            }
        }
    }
}
