/*
 * TestServer.cs
 *
 * Testing Framework
 * Version: 1.0
 *
 * Copyright (c) 2006, TopCoder, Inc. All rights reserved
 *
 * Author: TCSDEVELOPER
 */

using System.Net;
using NAnt.Core;
using NAnt.Core.Types;
using NUnit.Framework;

namespace TopCoder.Test.Framework.Servers.Tests
{
    /// <summary>
    /// Unit tests for the <see cref="Server"/> class.
    /// </summary>
    ///
    /// <remarks>
    /// Since <see cref="Server"/> is an abstract class,
    /// it is tested via the <see cref="MyServer"/> class.
    /// </remarks>
    [TestFixture]
    public class TestServer
    {
        /// <summary>
        /// The Server instance to test. Initialized in SetUp to MyServer instance.
        /// </summary>
        private Server server;

        /// <summary>
        /// A sample Option collection. Initialized in SetUp.
        /// </summary>
        private OptionCollection options;

        /// <summary>
        /// Initializes the <see cref="Server"/> instance to test via <see cref="MyServer"/>
        /// and prepares a sample option collection.
        /// </summary>
        [SetUp]
        public void SetUp()
        {
            server = new MyServer();

            // prepare a sample option collection
            options = new OptionCollection();

            // timeout
            Option optTimeOut = new Option();
            optTimeOut.OptionName = Server.TIMEOUT;
            optTimeOut.Value = "789";
            options.Add(optTimeOut);

            // port
            Option optPort = new Option();
            optPort.OptionName = Server.PORT;
            optPort.Value = "123";
            options.Add(optPort);

            // address
            Option optAddr = new Option();
            optAddr.OptionName = Server.ADDRESS;
            optAddr.Value = "software.topcoder.com";
            options.Add(optAddr);
        }

        /// <summary>
        /// Tests static constants definitions.
        /// </summary>
        [Test]
        public void TestStatic()
        {
            Assert.AreEqual("timeout", Server.TIMEOUT, "TIMEOUT value is incorrect");
            Assert.AreEqual("port", Server.PORT, "PORT value is incorrect");
            Assert.AreEqual("address", Server.ADDRESS, "ADDRESS value is incorrect");
        }

        /// <summary>
        /// Tests StartServer property. Make sure initial value is correct,
        /// then try setting and getting back the value.
        /// </summary>
        [Test]
        public void TestStartServer()
        {
            // default value
            Assert.IsTrue(server.StartServer, "Default value is incorrect");

            // set and assert
            server.StartServer = false;
            Assert.IsFalse(server.StartServer, "Changed to false");
        }

        /// <summary>
        /// Tests InitServer property. Make sure initial value is correct,
        /// then try setting and getting back the value.
        /// </summary>
        [Test]
        public void TestInitServer()
        {
            // default value
            Assert.IsTrue(server.InitServer, "Default value is incorrect");

            // set and assert
            server.InitServer = false;
            Assert.IsFalse(server.InitServer, "Changed to false");
        }

        /// <summary>
        /// Tests Options property. Make sure initial value is correct,
        /// then try setting and getting back the value.
        /// </summary>
        [Test]
        public void TestOptions()
        {
            // default value
            Assert.IsNull(server.Options, "Default value is incorrect");

            // set and assert
            server.Options = options;
            Assert.AreSame(options, server.Options, "Changed to sample options");

            // set to null
            server.Options = null;
            Assert.IsNull(server.Options, "Reset to null");
        }

        /// <summary>
        /// Tests Credentials property. Make sure initial value is correct,
        /// then try setting and getting back the value.
        /// </summary>
        [Test]
        public void TestCredentials()
        {
            // default value
            Assert.IsNull(server.Credentials, "Default value is incorrect");

            // set and assert
            Credential newCred = new Credential();
            server.Credentials = newCred;
            Assert.AreSame(newCred, server.Credentials, "Changed to a new credential");

            // set to null
            server.Credentials = null;
            Assert.IsNull(server.Credentials, "Reset to null");
        }

        /// <summary>
        /// Tests SourceFiles property. Make sure initial value is correct,
        /// then try setting and getting back the value.
        /// </summary>
        [Test]
        public void TestSourceFiles()
        {
            // default value
            Assert.IsNull(server.SourceFiles, "Default value is incorrect");

            // set and assert
            FileSet newSource = new FileSet();
            server.SourceFiles = newSource;
            Assert.AreSame(newSource, server.SourceFiles, "Changed to a new FileSet");

            // set to null
            server.SourceFiles = null;
            Assert.IsNull(server.SourceFiles, "Reset to null");
        }

        /// <summary>
        /// Tests Project property. Make sure initial value is correct,
        /// then try setting and getting back the value.
        /// </summary>
        [Test]
        public void TestProject()
        {
            // default value
            Assert.IsNull(server.Project, "Default value is incorrect");

            // set and assert
            Project project = new Project(@"..\..\test_files\ServerAppTest.build", Level.Error);
            server.Project = project;
            Assert.AreSame(project, server.Project, "Changed to a new Project");

            // set to null
            server.Project = null;
            Assert.IsNull(server.Project, "Reset to null");
        }

        /// <summary>
        /// Tests TimeOut property. Make sure it retrieves and parses OptionCollection correctly.
        /// </summary>
        [Test]
        public void TestTimeOut()
        {
            // set the options first and assert
            server.Options = options;
            Assert.AreEqual(789, server.Timeout, "TimeOut value as in the options");
        }

        /// <summary>
        /// Tests TimeOut property given Server with null option collection.
        /// Expecting ServerException.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ServerException))]
        public void TestTimeOutNullCollection()
        {
            int val = server.Timeout;
        }

        /// <summary>
        /// Tests TimeOut property given option collection that contains no timeout key.
        /// Expecting ServerException.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ServerException))]
        public void TestTimeOutNotExists()
        {
            // remove timeout from options
            options.Remove(options[Server.TIMEOUT]);
            server.Options = options;
            int val = server.Timeout;
        }

        /// <summary>
        /// Tests TimeOut property given option collection that contains illegal value (non-integer).
        /// Expecting ServerException.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ServerException))]
        public void TestTimeOutNonInteger()
        {
            // timeout is not an integer
            options[Server.TIMEOUT].Value = "abc";
            server.Options = options;
            int val = server.Timeout;
        }

        /// <summary>
        /// Tests TimeOut property given option collection that contains negative value.
        /// Expecting ServerException.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ServerException))]
        public void TestTimeOutNegative()
        {
            // timeout is negative
            options[Server.TIMEOUT].Value = "-1";
            server.Options = options;
            int val = server.Timeout;
        }

        /// <summary>
        /// Tests Port property. Make sure it retrieves and parses OptionCollection correctly.
        /// </summary>
        [Test]
        public void TestPort()
        {
            // set the options first and assert
            server.Options = options;
            Assert.AreEqual(123, server.Port, "Port value as in the options");
        }

        /// <summary>
        /// Tests Port property given Server with null option collection.
        /// Expecting ServerException.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ServerException))]
        public void TestPortNullCollection()
        {
            int val = server.Port;
        }

        /// <summary>
        /// Tests Port property given option collection that contains no port key.
        /// Expecting ServerException.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ServerException))]
        public void TestPortNotExists()
        {
            // remove port from options
            options.Remove(options[Server.PORT]);
            server.Options = options;
            int val = server.Port;
        }

        /// <summary>
        /// Tests Port property given option collection that contains illegal value (non-integer).
        /// Expecting ServerException.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ServerException))]
        public void TestPortNonInteger()
        {
            // port is not an integer
            options[Server.PORT].Value = "abc";
            server.Options = options;
            int val = server.Port;
        }

        /// <summary>
        /// Tests Port property given option collection that contains an integer
        /// less than IPEndPoint.MinPoint.
        /// Expecting ServerException.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ServerException))]
        public void TestPortNegative()
        {
            // port is negative
            options[Server.PORT].Value = (IPEndPoint.MinPort - 1).ToString();
            server.Options = options;
            int val = server.Port;
        }

        /// <summary>
        /// Tests Port property given option collection that contains an integer
        /// greater than IPEndPoint.MaxPort.
        /// Expecting ServerException.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ServerException))]
        public void TestPortBig()
        {
            // port is too big
            options[Server.PORT].Value = (IPEndPoint.MaxPort + 1).ToString();
            server.Options = options;
            int val = server.Port;
        }

        /// <summary>
        /// Tests Address property. Make sure it retrieves OptionCollection correctly.
        /// </summary>
        [Test]
        public void TestAddress()
        {
            // set the options first and assert
            server.Options = options;
            Assert.AreEqual("software.topcoder.com", server.Address, "Address value as in the options");
        }

        /// <summary>
        /// Tests Address property given Server with null option collection.
        /// Expecting ServerException.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ServerException))]
        public void TestAddressNullCollection()
        {
            string val = server.Address;
        }

        /// <summary>
        /// Tests Address property given option collection that contains no address key.
        /// Expecting ServerException.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ServerException))]
        public void TestAddressNotExists()
        {
            // remove address from options
            options.Remove(options[Server.ADDRESS]);
            server.Options = options;
            string val = server.Address;
        }
    }
}
