/*
 * TestServerException.cs
 *
 * Testing Framework
 * Version: 1.0
 *
 * Copyright (c) 2006, TopCoder, Inc. All rights reserved
 *
 * Author: TCSDEVELOPER
 */

using System;
using NUnit.Framework;

namespace TopCoder.Test.Framework.Servers.Tests
{
    /// <summary>
    /// Unit tests for the <see cref="ServerException"/> class.
    /// </summary>
    [TestFixture]
    public class TestServerException
    {
        /// <summary>
        /// A sample error message to use.
        /// </summary>
        private static string errMessage = "Error initializing server";

        /// <summary>
        /// A sample inner exceptions to use.
        /// </summary>
        private static Exception inner = new Exception();

        /// <summary>
        /// Tests constructor(string) accuracy.
        /// </summary>
        [Test]
        public void TestCtorString()
        {
            // instantiate with a sample error message
            ServerException e = new ServerException(errMessage);

            // assert error message is correct and inner exception is null
            Assert.AreEqual(errMessage, e.Message, "The error message is incorrect");
            Assert.IsNull(e.InnerException, "No inner exception");
        }

        /// <summary>
        /// Tests constructor(string) given null error message.
        /// Expecting a default error message in the exception.
        /// </summary>
        [Test]
        public void TestCtorStringNull()
        {
            // instantiate with null error message
            ServerException e = new ServerException(null);

            // assert default error message and inner exception is null
            Assert.IsNotNull(e.Message, "The error message is initialized to default");
            Assert.IsNull(e.InnerException, "No inner exception");
        }

        /// <summary>
        /// Tests constructor(string) given empty error message.
        /// Expecting an empty error message and no exception is thrown.
        /// </summary>
        [Test]
        public void TestCtorStringEmpty()
        {
            // instantiate with empty error message
            ServerException e = new ServerException(" ");

            // assert empty error message and inner exception is null
            Assert.AreEqual(" ", e.Message, "The error message is empty");
            Assert.IsNull(e.InnerException, "No inner exception");
        }

        /// <summary>
        /// Tests constructor(string, Exception) accuracy.
        /// </summary>
        [Test]
        public void TestCtorStringException()
        {
            // instantiate with a sample error message and inner exception
            ServerException e = new ServerException(errMessage, inner);

            // assert error message and inner exception is correct
            Assert.AreEqual(errMessage, e.Message, "The error message is incorrect");
            Assert.AreSame(inner, e.InnerException, "Inner exception is incorrect");
        }

        /// <summary>
        /// Tests constructor(string, Exception) with a null error message and null inner exceptions.
        /// Expecting a default error message in the exception and null inner exceptions.
        /// </summary>
        [Test]
        public void TestCtorStringExceptionNull()
        {
            // instantiate with null error message and inner exceptions
            ServerException e = new ServerException(null, null);

            // assert error message is initialized to default message and inner exceptions are null
            Assert.IsNotNull(e.Message, "The error message is the default message");
            Assert.IsNull(e.InnerException, "No inner exception");
        }

        /// <summary>
        /// Tests constructor(string, Exception) with an empty error message.
        /// Expecting an empty error message and no exception is thrown.
        /// </summary>
        [Test]
        public void TestCtorStringExceptionEmpty()
        {
            // instantiate with an empty error message
            ServerException e = new ServerException(" ", inner);

            // assert error message and inner exception are correct
            Assert.AreEqual(" ", e.Message, "The error message is empty");
            Assert.AreSame(inner, e.InnerException, "Inner exception is incorrect");
        }

        /// <summary>
        /// Tests <see cref="ServerException"/> inherits from <see cref="ApplicationException"/>.
        /// </summary>
        [Test]
        public void TestInheritance()
        {
            try
            {
                throw new ServerException(errMessage, inner);
            }
            catch (Exception e)
            {
                // ok only if it's a direct subclass of ApplicationException
                Assert.AreEqual(typeof(ApplicationException), e.GetType().BaseType,
                    "ServerException is not a direct subclass of ApplicationException");
            }
        }
    }
}
