/*
 * TestServerAppTestTask.cs
 *
 * Testing Framework
 * Version: 1.0
 *
 * Copyright (c) 2006, TopCoder, Inc. All rights reserved
 *
 * Author: TCSDEVELOPER
 */

using System.Xml;
using NAnt.Core;
using NAnt.Core.Attributes;
using NAnt.Core.Types;
using NUnit.Framework;
using TopCoder.Test.Framework.Servers;
using TopCoder.Test.Framework.Servers.Tests;

namespace TopCoder.Test.Framework.Tests
{
    /// <summary>
    /// Unit tests for the <see cref="ServerAppTestTask"/> class.
    /// </summary>
    ///
    /// <remarks>
    /// First the getter and setter is tested, then
    /// it's tested as if it's run via NAnt.
    /// </remarks>
    [TestFixture]
    public class TestServerAppTestTask
    {
        /// <summary>
        /// The ServerAppTestTask instance to test. Initialized in SetUp.
        /// </summary>
        private ServerAppTestTask task;

        /// <summary>
        /// A sample ServerElement instance. Initialized in SetUp.
        /// </summary>
        private ServerElement element;

        /// <summary>
        /// A second sample of ServerElement instance. Initialized in SetUp.
        /// </summary>
        private ServerElement element2;

        /// <summary>
        /// A third sample of ServerElement instance. Initialized in SetUp.
        /// </summary>
        private ServerElement element3;

        /// <summary>
        /// Initializes the <see cref="ServerAppTestTask"/> instance to test
        /// and prepares a sample ServerElement.
        /// </summary>
        [SetUp]
        public void SetUp()
        {
            task = new ServerAppTestTask();

            // prepare a sample ServerElement to MyServer
            element = new ServerElement();
            element.ClassName = typeof(MyServer).ToString();
            element.StartServer = true;
            element.InitServer = true;

            // prepare second and third sample
            element2 = new ServerElement();
            element2.ClassName = element.ClassName;
            element2.StartServer = true;
            element2.InitServer = true;

            element3 = new ServerElement();
            element3.ClassName = element.ClassName;
            element3.StartServer = true;
            element3.InitServer = true;

            // prepare an empty dummy project and assign to the task
            // because task cannot be executed until a Project is assigned
            XmlDocument doc = new XmlDocument();
            doc.LoadXml("<project/>");
            task.Project = new Project(doc, Level.Error);
        }

        /// <summary>
        /// Tests Servers property. Make sure initial value is correct,
        /// then try setting and getting back the value.
        /// Also assert that a BuildElementArray is attached.
        /// </summary>
        [Test]
        public void TestServers()
        {
            // default value
            Assert.IsNull(task.Servers, "Default value is incorrect");

            // set and assert
            ServerElement[] element = new ServerElement[5];
            task.Servers = element;
            Assert.AreSame(element, task.Servers, "Changed to a new ServerElement[]");

            // set to null
            task.Servers = null;
            Assert.IsNull(task.Servers, "Reset to null");

            // assert attribute
            object[] attr = typeof(ServerAppTestTask).GetProperty("Servers").GetCustomAttributes(false);
            Assert.AreEqual(1, attr.Length, "1 attribute is attached");
            Assert.IsTrue(attr[0] is BuildElementArrayAttribute, "Servers is attributed");
            Assert.AreEqual("server", ((BuildElementArrayAttribute) attr[0]).Name,
                "server is the element name");
            Assert.IsTrue(((BuildElementArrayAttribute) attr[0]).Required, "server is required");
        }

        /// <summary>
        /// Tests with NAnt Project given a task with 1 server defined.
        /// All non-required elements are omitted and a default is expected.
        /// Expecting that 1 ServerElement is created.
        /// </summary>
        [Test]
        public void TestNAnt1()
        {
            // create a sample NAnt project file
            // only 1 server and only required class attribute is given
            XmlDocument doc = new XmlDocument();
            doc.LoadXml(string.Format(
                @"<project>
                    <serverapptest>
                        <server class=""Minimal""/>
                    </serverapptest>
                </project>", typeof(ServerAppTestTask).Assembly.Location));

            // create a new project and initialize the ServerAppTestTask with the xml
            task.Project = new Project(doc, Level.Error);
            task.Initialize(doc.SelectSingleNode("/project/serverapptest"));

            // assert xml is parsed correctly: 1 Server is parsed
            Assert.AreEqual(1, task.Servers.Length, "1 server is registered");
            Assert.AreEqual("Minimal", task.Servers[0].ClassName, "'class' is not parsed");
            Assert.IsTrue(task.Servers[0].StartServer, "StartServer should be the default");
            Assert.IsTrue(task.Servers[0].InitServer, "InitServer should be the default");
            Assert.AreEqual(0, task.Servers[0].Options.Count, "Options should be the default");
            Assert.IsNull(task.Servers[0].Credentials.UserName, "Credentials should be the default");
            Assert.AreEqual(0, task.Servers[0].SourceFiles.FileNames.Count, "SourceFiles should be the default");
        }

        /// <summary>
        /// Tests with NAnt Project given a task with 2 servers defined.
        /// Expecting that 2 ServerElement are parsed.
        /// </summary>
        [Test]
        public void TestNAnt2()
        {
            // create a sample NAnt project file
            // only 1 server and only required class attribute is given
            XmlDocument doc = new XmlDocument();
            doc.LoadXml(string.Format(
                @"<project>
                    <serverapptest>
                        <server class=""First"" startserver=""false"">
                            <options>
                                <option name=""Server1"" value=""Opt1""/>
                            </options>
                        </server>
                        <server class=""Second"" initserver=""false"">
                            <credentials username=""S2"" password=""PWD2""/>
                        </server>
                    </serverapptest>
                </project>", typeof(ServerAppTestTask).Assembly.Location));

            // create a new project and initialize the ServerAppTestTask with the xml
            task.Project = new Project(doc, Level.Error);
            task.Initialize(doc.SelectSingleNode("/project/serverapptest"));

            // assert xml is parsed correctly: 2 Servers are parsed
            Assert.AreEqual(2, task.Servers.Length, "2 servers are registered");

            // 1st server
            Assert.AreEqual("First", task.Servers[0].ClassName, "1st 'class' is not parsed");
            Assert.IsFalse(task.Servers[0].StartServer, "1st StartServer is false");
            Assert.IsTrue(task.Servers[0].InitServer, "1st InitServer should be the default");
            Assert.AreEqual(1, task.Servers[0].Options.Count, "1st Options has 1 option");
            Assert.AreEqual("Opt1", task.Servers[0].Options["Server1"].Value, "1st Option value");
            Assert.IsNull(task.Servers[0].Credentials.UserName, "1st Credentials should be the default");
            Assert.AreEqual(0, task.Servers[0].SourceFiles.FileNames.Count,
                "1st SourceFiles should be the default");

            // 2nd server
            Assert.AreEqual("Second", task.Servers[1].ClassName, "2nd 'class' is not parsed");
            Assert.IsTrue(task.Servers[1].StartServer, "2nd StartServer should be the default");
            Assert.IsFalse(task.Servers[1].InitServer, "2nd InitServer is false");
            Assert.AreEqual(0, task.Servers[1].Options.Count, "2nd Options not exists");
            Assert.AreEqual("S2", task.Servers[1].Credentials.UserName, "2nd Credentials's UserName");
            Assert.AreEqual("PWD2", task.Servers[1].Credentials.Password, "2nd Credentials's Password");
            Assert.AreEqual(0, task.Servers[1].SourceFiles.FileNames.Count,
                "2nd SourceFiles should be the default");
        }

        /// <summary>
        /// Tests Execute given a prepared ServerAppTestTask.
        /// Expecting that the server methods are executed in the correct order.
        /// </summary>
        [Test]
        public void TestExecute()
        {
            // prepare 1 server and execute
            task.Servers = new ServerElement[] { element };
            task.Execute();

            // assert that methods are called in the correct order
            MyServer s = (MyServer) element.GetServerInstance();
            Assert.IsTrue(s.TickStartUp != -1, "Server is started");
            Assert.IsTrue(s.TickInitData > s.TickStartUp, "Started then initialized");
            Assert.IsTrue(s.TickCleanUpData > s.TickInitData, "Initialized then cleaned");
            Assert.IsTrue(s.TickShutDown > s.TickCleanUpData, "Cleaned then stopped");
        }

        /// <summary>
        /// Tests Execute given an element which has its startserver and initserver
        /// both set to false.
        /// </summary>
        [Test]
        public void TestExecuteFalse()
        {
            // prepare 1 server (don't start) and execute
            element.StartServer = false;
            element.InitServer = false;
            task.Servers = new ServerElement[] { element };
            task.Execute();

            // assert that methods are not called
            MyServer s = (MyServer) element.GetServerInstance();
            Assert.AreEqual(-1, s.TickStartUp, "Server not started");
            Assert.AreEqual(-1, s.TickInitData, "Server not initialized");
            Assert.AreEqual(-1, s.TickCleanUpData, "Server not cleaned");
            Assert.AreEqual(-1, s.TickShutDown, "Server not stopped");
        }

        /// <summary>
        /// Tests Execute given two servers.
        /// First server should not be started, but initialized.
        /// Second server should be started, but not initialized.
        /// </summary>
        [Test]
        public void TestExecuteTwoServers()
        {
            // prepare servers and execute
            element.StartServer = false;
            element2.InitServer = false;
            task.Servers = new ServerElement[] { element, element2 };
            task.Execute();

            // assert 1st server methods are called correctly
            MyServer s = (MyServer) element.GetServerInstance();
            Assert.AreEqual(-1, s.TickStartUp, "Server1 not started");
            Assert.IsTrue(s.TickInitData != -1, "Server1 initialized");
            Assert.IsTrue(s.TickCleanUpData > s.TickInitData, "Server 1 cleaned");
            Assert.AreEqual(-1, s.TickShutDown, "Server1 not stopped");

            // assert 2nd server methods are called correctly
            MyServer s2 = (MyServer) element2.GetServerInstance();
            Assert.IsTrue(s2.TickStartUp != -1, "Server2 started");
            Assert.AreEqual(-1, s2.TickInitData, "Server2 not initialized");
            Assert.AreEqual(-1, s2.TickCleanUpData, "Server2 not cleaned");
            Assert.IsTrue(s2.TickShutDown > s2.TickStartUp, "Server2 stopped");
        }

        /// <summary>
        /// Tests Execute given three servers to be started/initialized.
        /// However second server will fail while starting. ServerTaskException should be thrown.
        /// First server will be started, but not initialized yet, then stopped.
        /// Second server will be started, failed, and then not initialized.
        /// Third server will not be touched.
        /// </summary>
        [Test]
        public void TestExecuteThreeServersStartException()
        {
            try
            {
                // server2 should throw exception on Start
                Option startException = new Option();
                startException.OptionName = MyServer.StartUpException;
                startException.Value = "true";
                element2.Options.Add(startException);

                // execute the task
                task.Servers = new ServerElement[] { element, element2, element3 };
                task.Execute();
                Assert.Fail("ServerTaskException should be thrown by server2 startup");
            }
            catch (ServerTaskException e)
            {
                // good, assert what server was started/initialized/cleaned/stopped

                // assert 1st server methods are started but not initialized
                MyServer s = (MyServer) element.GetServerInstance();
                Assert.IsTrue(s.TickStartUp != -1, "Server1 is started");
                Assert.AreEqual(-1, s.TickInitData, "Server1 not initialized");
                Assert.AreEqual(-1, s.TickCleanUpData, "Server1 not cleaned");
                Assert.IsTrue(s.TickShutDown > s.TickStartUp, "Server1 stopped");

                // assert 2nd server methods are started but failed, then stopped
                s = (MyServer) element2.GetServerInstance();
                Assert.IsTrue(s.TickStartUp != -1, "Server2 started but failed");
                Assert.AreEqual(-1, s.TickInitData, "Server2 not initialized");
                Assert.AreEqual(-1, s.TickCleanUpData, "Server2 not cleaned");
                Assert.AreEqual(-1, s.TickShutDown, "Server2 not stopped");

                // assert 3nd server methods are not touched at all
                s = (MyServer) element3.GetServerInstance();
                Assert.AreEqual(-1, s.TickStartUp, "Server3 not started");
                Assert.AreEqual(-1, s.TickInitData, "Server3 not initialized");
                Assert.AreEqual(-1, s.TickCleanUpData, "Server3 not cleaned");
                Assert.AreEqual(-1, s.TickShutDown, "Server3 not stopped");

                // assert inner exception contains 1 ServerException
                Assert.AreEqual(1, e.InnerExceptions.Length, "1 inner exception");
                Assert.IsTrue(e.InnerExceptions[0] is ServerException, "Inner exception is ServerException");
            }
        }

        /// <summary>
        /// Tests Execute given three servers to be started/initialized.
        /// However second server will fail while initializing. ServerTaskException should be thrown.
        /// All servers will be started; then initialization fails on second server.
        /// So third server never gets initialized.
        /// </summary>
        [Test]
        public void TestExecuteThreeServersInitException()
        {
            try
            {
                // server2 should throw exception on Init
                Option initException = new Option();
                initException.OptionName = MyServer.InitDataException;
                initException.Value = "true";
                element2.Options.Add(initException);

                // execute the task
                task.Servers = new ServerElement[] { element, element2, element3 };
                task.Execute();
                Assert.Fail("ServerTaskException should be thrown by server2 init");
            }
            catch (ServerTaskException e)
            {
                // good, assert what server was started/initialized/cleaned/stopped

                // assert 1st server methods are all called
                MyServer s = (MyServer) element.GetServerInstance();
                Assert.IsTrue(s.TickStartUp != -1, "Server1 is started");
                Assert.IsTrue(s.TickInitData > s.TickStartUp, "Server1 initialized");
                Assert.IsTrue(s.TickCleanUpData > s.TickInitData, "Server1 cleaned");
                Assert.IsTrue(s.TickShutDown > s.TickCleanUpData, "Server1 stopped");

                // assert 2nd server methods are called correctly
                s = (MyServer) element2.GetServerInstance();
                Assert.IsTrue(s.TickStartUp != -1, "Server2 started");
                Assert.IsTrue(s.TickInitData != -1, "Server2 initialized, but failed");
                Assert.AreEqual(-1, s.TickCleanUpData, "Server2 not cleaned");
                Assert.IsTrue(s.TickShutDown > s.TickStartUp, "Server2 stopped");

                // assert 3nd server methods are started, but not initialized
                s = (MyServer) element3.GetServerInstance();
                Assert.IsTrue(s.TickStartUp != -1, "Server3 started");
                Assert.AreEqual(-1, s.TickInitData, "Server3 not initialized");
                Assert.AreEqual(-1, s.TickCleanUpData, "Server3 not cleaned");
                Assert.IsTrue(s.TickShutDown > s.TickStartUp, "Server3 stopped");

                // assert inner exception contains 1 ServerException
                Assert.AreEqual(1, e.InnerExceptions.Length, "1 inner exception");
                Assert.IsTrue(e.InnerExceptions[0] is ServerException, "Inner exception is ServerException");
            }
        }

        /// <summary>
        /// Tests Execute given three servers to be started/initialized.
        /// However second server will fail while cleaning-up.
        /// And third server will fail while shutting down. ServerTaskException should be thrown.
        /// Nevertheless, all server should still be started/initialized/cleaned/stopped.
        /// The inner exceptions will have 2 items.
        /// </summary>
        [Test]
        public void TestExecuteThreeServersCleanShutdownException()
        {
            try
            {
                // server2 should throw exception on CleanUp
                Option cleanException = new Option();
                cleanException.OptionName = MyServer.CleanUpDataException;
                cleanException.Value = "true";
                element2.Options.Add(cleanException);

                // server3 throw exception on shut down
                Option closeException = new Option();
                closeException.OptionName = MyServer.ShutDownException;
                closeException.Value = "true";
                element3.Options.Add(closeException);

                // execute the task
                task.Servers = new ServerElement[] { element, element2, element3 };
                task.Execute();
                Assert.Fail("ServerTaskException should be thrown by server2/3");
            }
            catch (ServerTaskException e)
            {
                // good, assert what server was started/initialized/cleaned/stopped

                // assert 1st server methods are all called
                MyServer s = (MyServer) element.GetServerInstance();
                Assert.IsTrue(s.TickStartUp != -1, "Server1 is started");
                Assert.IsTrue(s.TickInitData > s.TickStartUp, "Server1 initialized");
                Assert.IsTrue(s.TickCleanUpData > s.TickInitData, "Server1 cleaned");
                Assert.IsTrue(s.TickShutDown > s.TickCleanUpData, "Server1 stopped");

                // assert 2nd server methods are all called although fail on cleaning up
                s = (MyServer) element2.GetServerInstance();
                Assert.IsTrue(s.TickStartUp != -1, "Server2 is started");
                Assert.IsTrue(s.TickInitData > s.TickStartUp, "Server2 initialized");
                Assert.IsTrue(s.TickCleanUpData > s.TickInitData, "Server2 cleaned although failed");
                Assert.IsTrue(s.TickShutDown > s.TickCleanUpData, "Server2 stopped");

                // assert 3nd server methods are all called altough fail on shutting down
                s = (MyServer) element3.GetServerInstance();
                Assert.IsTrue(s.TickStartUp != -1, "Server3 is started");
                Assert.IsTrue(s.TickInitData > s.TickStartUp, "Server3 initialized");
                Assert.IsTrue(s.TickCleanUpData > s.TickInitData, "Server3 cleaned");
                Assert.IsTrue(s.TickShutDown > s.TickCleanUpData, "Server2 stopped although failed");

                // assert inner exception contains 2 ServerException
                Assert.AreEqual(2, e.InnerExceptions.Length, "2 inner exception");
                Assert.IsTrue(e.InnerExceptions[0] is ServerException, "From server2");
                Assert.IsTrue(e.InnerExceptions[1] is ServerException, "And from server3");
            }
        }

        /// <summary>
        /// Tests Execute with a sample NAnt Xml.
        /// </summary>
        [Test]
        public void TestExecuteNAnt()
        {
            // create a sample NAnt project file
            XmlDocument doc = new XmlDocument();
            doc.LoadXml(string.Format(
                @"<project default=""test"">
                    <loadtasks assembly=""{0}""/>
                    <target name=""test"">
                        <serverapptest>
                            <server class=""TopCoder.Test.Framework.Servers.Tests.MyServer""
                                startserver=""true"" initserver=""true"">
                                <options>
                                    <option name=""port"" value=""123""/>
                                </options>
                            </server>
                            <test assemblyname=""NotExists.dll""/>
                        </serverapptest>
                    </target>
                </project>", typeof(MyServer).Assembly.Location));

            // create a new project and initialize the ServerAppTestTask with the xml
            Project p = new Project(doc, Level.Info);

            using (ConsoleCap c = new ConsoleCap())
            {
                p.Run();

                // assert 1 target executed
                Assert.IsTrue(((Target) p.Targets[0]).Executed, "Target executed");

                // assert that NUnit test failed because the assembly doesn't exist
                Assert.IsTrue(c.Output.IndexOf("NotExists") >= 0,
                    "NUnit should fail: " + c.Output);
            }
        }
    }
}
