/*
 * TestServerElement.cs
 *
 * Testing Framework
 * Version: 1.0
 *
 * Copyright (c) 2006, TopCoder, Inc. All rights reserved
 *
 * Author: TCSDEVELOPER
 */

using System.Xml;
using NAnt.Core;
using NAnt.Core.Attributes;
using NAnt.Core.Types;
using NUnit.Framework;
using TopCoder.Test.Framework.Servers;
using TopCoder.Test.Framework.Servers.Tests;

namespace TopCoder.Test.Framework.Tests
{
    /// <summary>
    /// Unit tests for the <see cref="ServerElement"/> class.
    /// </summary>
    ///
    /// <remarks>
    /// First all properties getters and setters are tested, then
    /// it's tested as if it's run via NAnt.
    /// </remarks>
    [TestFixture]
    public class TestServerElement
    {
        /// <summary>
        /// The ServerElement instance to test. Initialized in SetUp.
        /// </summary>
        private ServerElement serverElement;

        /// <summary>
        /// Initializes the <see cref="ServerElement"/> instance to test.
        /// </summary>
        [SetUp]
        public void SetUp()
        {
            serverElement = new ServerElement();
        }

        /// <summary>
        /// Tests ClassName property. Make sure initial value is correct,
        /// then try setting and getting back the value.
        /// Also assert that a TaskAttribute is attached.
        /// </summary>
        [Test]
        public void TestClassName()
        {
            // default value
            Assert.IsNull(serverElement.ClassName, "Default value is incorrect");

            // set and assert
            serverElement.ClassName = "NewClass";
            Assert.AreEqual("NewClass", serverElement.ClassName, "Changed to NewClass");

            // set to null
            serverElement.ClassName = null;
            Assert.IsNull(serverElement.ClassName, "Reset to null");

            // assert attribute
            object[] attr = typeof(ServerElement).GetProperty("ClassName").GetCustomAttributes(false);
            Assert.AreEqual(1, attr.Length, "1 attribute is attached");
            Assert.IsTrue(attr[0] is TaskAttributeAttribute, "ClassName is attributed");
            Assert.IsTrue(((TaskAttributeAttribute) attr[0]).Required, "ClassName is required");
        }

        /// <summary>
        /// Tests StartServer property. Make sure initial value is correct,
        /// then try setting and getting back the value.
        /// Also assert that a TaskAttribute is attached.
        /// </summary>
        [Test]
        public void TestStartServer()
        {
            // default value
            Assert.IsTrue(serverElement.StartServer, "Default value is incorrect");

            // set and assert
            serverElement.StartServer = false;
            Assert.IsFalse(serverElement.StartServer, "Changed to false");

            // assert attribute
            object[] attr = typeof(ServerElement).GetProperty("StartServer").GetCustomAttributes(false);
            Assert.AreEqual(1, attr.Length, "1 attribute is attached");
            Assert.IsTrue(attr[0] is TaskAttributeAttribute, "StartServer is attributed");
            Assert.IsFalse(((TaskAttributeAttribute) attr[0]).Required, "StartServer is not required");
        }

        /// <summary>
        /// Tests InitServer property. Make sure initial value is correct,
        /// then try setting and getting back the value.
        /// Also assert that a TaskAttribute is attached.
        /// </summary>
        [Test]
        public void TestInitServer()
        {
            // default value
            Assert.IsTrue(serverElement.InitServer, "Default value is incorrect");

            // set and assert
            serverElement.InitServer = false;
            Assert.IsFalse(serverElement.InitServer, "Changed to false");

            // assert attribute
            object[] attr = typeof(ServerElement).GetProperty("InitServer").GetCustomAttributes(false);
            Assert.AreEqual(1, attr.Length, "1 attribute is attached");
            Assert.IsTrue(attr[0] is TaskAttributeAttribute, "InitServer is attributed");
            Assert.IsFalse(((TaskAttributeAttribute) attr[0]).Required, "InitServer is not required");
        }

        /// <summary>
        /// Tests Options property. Make sure initial value is correct.
        /// Also assert that a BuildElementCollection is attached.
        /// </summary>
        [Test]
        public void TestOptions()
        {
            // default value
            Assert.IsNotNull(serverElement.Options, "Default value is incorrect");
            Assert.AreEqual(0, serverElement.Options.Count, "No options initially");

            // assert attribute
            object[] attr = typeof(ServerElement).GetProperty("Options").GetCustomAttributes(false);
            Assert.AreEqual(1, attr.Length, "1 attribute is attached");
            Assert.IsTrue(attr[0] is BuildElementCollectionAttribute, "Options is attributed");
            Assert.AreEqual("options", ((BuildElementCollectionAttribute) attr[0]).Name,
                "options is the element name");
            Assert.AreEqual("option", ((BuildElementCollectionAttribute) attr[0]).ChildElementName,
                "option is child element");
            Assert.IsFalse(((BuildElementCollectionAttribute) attr[0]).Required, "Options is not required");
        }

        /// <summary>
        /// Tests Credentials property. Make sure initial value is correct,
        /// then try setting and getting back the value.
        /// Also assert that a BuildElement is attached.
        /// </summary>
        [Test]
        public void TestCredentials()
        {
            // default value
            Assert.IsNotNull(serverElement.Credentials, "Default value is incorrect");
            Assert.IsNull(serverElement.Credentials.UserName, "No name");

            // set and assert
            Credential newCred = new Credential();
            serverElement.Credentials = newCred;
            Assert.AreSame(newCred, serverElement.Credentials, "Changed to a new credential");

            // set to null
            serverElement.Credentials = null;
            Assert.IsNull(serverElement.Credentials, "Reset to null");

            // assert attribute
            object[] attr = typeof(ServerElement).GetProperty("Credentials").GetCustomAttributes(false);
            Assert.AreEqual(1, attr.Length, "1 attribute is attached");
            Assert.IsTrue(attr[0] is BuildElementAttribute, "Credentials is attributed");
            Assert.AreEqual("credentials", ((BuildElementAttribute) attr[0]).Name,
                "credentials is the element name");
            Assert.IsFalse(((BuildElementAttribute) attr[0]).Required, "Credentials is not required");
        }

        /// <summary>
        /// Tests SourceFiles property. Make sure initial value is correct,
        /// then try setting and getting back the value.
        /// Also assert that a BuildElement is attached.
        /// </summary>
        [Test]
        public void TestSourceFiles()
        {
            // default value
            Assert.IsNotNull(serverElement.SourceFiles, "Default value is incorrect");
            Assert.AreEqual(0, serverElement.SourceFiles.FileNames.Count, "No files");

            // set and assert
            FileSet newSource = new FileSet();
            serverElement.SourceFiles = newSource;
            Assert.AreSame(newSource, serverElement.SourceFiles, "Changed to a new FileSet");

            // set to null
            serverElement.SourceFiles = null;
            Assert.IsNull(serverElement.SourceFiles, "Reset to null");

            // assert attribute
            object[] attr = typeof(ServerElement).GetProperty("SourceFiles").GetCustomAttributes(false);
            Assert.AreEqual(1, attr.Length, "1 attribute is attached");
            Assert.IsTrue(attr[0] is BuildElementAttribute, "SourceFiles is attributed");
            Assert.AreEqual("sourcefiles", ((BuildElementAttribute) attr[0]).Name,
                "sourcefiles is the element name");
            Assert.IsFalse(((BuildElementAttribute) attr[0]).Required, "SourceFiles is not required");
        }

        /// <summary>
        /// Tests with NAnt Project given a minimal (required-only) NAnt Xml.
        /// All non-required elements are omitted and a default is expected.
        /// </summary>
        [Test]
        public void TestNAntMinimal()
        {
            // create a sample NAnt project file (only required class attribute is given)
            XmlDocument doc = new XmlDocument();
            doc.LoadXml(@"<project><server class=""Minimal""/></project>");

            // create a new project and initialize the ServerElement with the xml
            serverElement.Project = new Project(doc, Level.Error);
            serverElement.Initialize(doc.SelectSingleNode("/project/server"));

            // assert ClassName is parsed, all other elements should be the default values
            Assert.AreEqual("Minimal", serverElement.ClassName, "'class' is not parsed");
            Assert.IsTrue(serverElement.StartServer, "StartServer should be the default");
            Assert.IsTrue(serverElement.InitServer, "InitServer should be the default");
            Assert.AreEqual(0, serverElement.Options.Count, "Options should be the default");
            Assert.IsNull(serverElement.Credentials.UserName, "Credentials should be the default");
            Assert.AreEqual(0, serverElement.SourceFiles.FileNames.Count, "SourceFiles should be the default");
        }

        /// <summary>
        /// Tests with NAnt Project given a full NAnt Xml.
        /// All elements are set.
        /// </summary>
        [Test]
        public void TestNAnt()
        {
            // create a sample NAnt project file
            XmlDocument doc = new XmlDocument();
            doc.LoadXml(
                @"<project>
                    <server class=""SampleServer"" startserver=""false"" initserver=""false"">
                        <options>
                            <option name=""Opt1"" value=""Opt1Value""/>
                            <option name=""TCS"" value=""TCO06""/>
                        </options>
                        <credentials username=""credUser"" password=""credPwd""/>
                        <sourcefiles>
                            <includes name=""..\..\test_files\*""/>
                            <excludes name=""NotExists""/>
                        </sourcefiles>
                    </server>
                </project>");

            // create a new project and initialize the ServerElement with the xml
            serverElement.Project = new Project(doc, Level.Error);
            serverElement.Initialize(doc.SelectSingleNode("/project/server"));

            // assert xml is parsed correctly
            Assert.AreEqual("SampleServer", serverElement.ClassName, "'class' is not parsed");
            Assert.IsFalse(serverElement.StartServer, "startserver is set to false");
            Assert.IsFalse(serverElement.InitServer, "initserver is set to false");

            Assert.AreEqual(2, serverElement.Options.Count, "2 Options are defined");
            Assert.AreEqual("Opt1Value", serverElement.Options["Opt1"].Value, "Option Opt1");
            Assert.AreEqual("TCO06", serverElement.Options["TCS"].Value, "Option TCS");

            Assert.AreEqual("credUser", serverElement.Credentials.UserName, "'username' is not parsed");
            Assert.AreEqual("credPwd", serverElement.Credentials.Password, "'password' is not parsed");

            Assert.IsTrue(serverElement.SourceFiles.FileNames.Count > 0, "SourceFiles should contain some files");
            Assert.AreEqual(@"..\..\test_files\*", serverElement.SourceFiles.Includes[0], "SourceFiles includes");
            Assert.AreEqual("NotExists", serverElement.SourceFiles.Excludes[0], "SourceFiles excludes");
        }

        /// <summary>
        /// Tests GetServerInstance given a prepared ServerElement.
        /// Expecting that the returned server has correct properties copied as well.
        /// </summary>
        [Test]
        public void TestGetServerInstance()
        {
            // change boolean properties to some non-default value
            serverElement.ClassName = typeof(MyServer).ToString();
            serverElement.StartServer = false;
            serverElement.InitServer = false;

            // get the instance
            Server server = serverElement.GetServerInstance();

            // assert that server properties are initialized
            Assert.IsFalse(server.StartServer, "StartServer is initialized");
            Assert.IsFalse(server.InitServer, "InitServer is initialized");
            Assert.AreSame(serverElement.Credentials, server.Credentials, "Credentials is initialized");
            Assert.AreSame(serverElement.Options, server.Options, "Options is initialized");
            Assert.AreSame(serverElement.SourceFiles, server.SourceFiles, "SourceFiles is initialized");
            Assert.AreSame(serverElement.Project, server.Project, "Project is copied");
        }

        /// <summary>
        /// Tests invoking GetServerInstance twice returns the same instance.
        /// </summary>
        [Test]
        public void TestGetServerInstanceCached()
        {
            // use MyServer as ClassName (use the fully qualified assembly name)
            serverElement.ClassName = typeof(MyServer).AssemblyQualifiedName;

            // get the instance twice
            Server server = serverElement.GetServerInstance();
            Server server2 = serverElement.GetServerInstance();

            Assert.AreSame(server, server2, "Same instance should be returned");
        }

        /// <summary>
        /// Tests GetServerInstance given a ClassName which is not of type Server.
        /// Expecting ServerException.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ServerException))]
        public void TestGetServerInstanceNotServer()
        {
            // use String
            serverElement.ClassName = typeof(ServerElement).ToString();

            // get the instance twice
            serverElement.GetServerInstance();
        }

        /// <summary>
        /// Tests GetServerInstance given default null ClassName.
        /// Expecting ServerException.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ServerException))]
        public void TestGetServerInstanceNullClassName()
        {
            serverElement.GetServerInstance();
        }

        /// <summary>
        /// Tests GetServerInstance with a full NAnt Xml.
        /// All properties should be set as specified in the xml.
        /// </summary>
        [Test]
        public void TestGetServerInstanceNAnt()
        {
            // create a sample NAnt project file
            XmlDocument doc = new XmlDocument();
            doc.LoadXml(
                @"<project>
                    <server class=""TopCoder.Test.Framework.Servers.Tests.MyApplicationServer""
                            startserver=""false"" initserver=""false"">
                        <options>
                            <option name=""deployment_dir"" value=""..\..\test_files""/>
                            <option name=""port"" value=""123""/>
                        </options>
                        <sourcefiles>
                            <includes name=""..\..\test_files\*""/>
                            <excludes name=""NotExists""/>
                        </sourcefiles>
                    </server>
                </project>");

            // create a new project and initialize the ServerElement with the xml
            serverElement.Project = new Project(doc, Level.Error);
            serverElement.Initialize(doc.SelectSingleNode("/project/server"));

            // get the server instance and assert properties
            Server server = serverElement.GetServerInstance();
            Assert.IsTrue(server is MyApplicationServer, "'class' is not parsed correctly");
            Assert.IsFalse(server.StartServer, "startserver is set to false");
            Assert.IsFalse(server.InitServer, "initserver is set to false");

            Assert.AreEqual(2, server.Options.Count, "2 Options are defined");
            Assert.AreEqual(123, ((MyApplicationServer) server).Port, "Server port");
            Assert.AreEqual(@"..\..\test_files", ((MyApplicationServer) server).DeploymentDirectory,
                "Server deployment_dir");

            Assert.IsTrue(server.SourceFiles.FileNames.Count > 0, "SourceFiles should contain some files");
            Assert.AreEqual(@"..\..\test_files\*", server.SourceFiles.Includes[0], "SourceFiles includes");
            Assert.AreEqual("NotExists", server.SourceFiles.Excludes[0], "SourceFiles excludes");

            Assert.AreSame(serverElement.Project, server.Project, "Project is copied");
        }
    }
}
