/*
 * TestServerTaskException.cs
 *
 * Testing Framework
 * Version: 1.0
 *
 * Copyright (c) 2006, TopCoder, Inc. All rights reserved
 *
 * Author: TCSDEVELOPER
 */

using System;
using NAnt.Core;
using NUnit.Framework;

namespace TopCoder.Test.Framework.Tests
{
    /// <summary>
    /// Unit tests for the <see cref="ServerTaskException"/> class.
    /// </summary>
    [TestFixture]
    public class TestServerTaskException
    {
        /// <summary>
        /// A sample error message to use.
        /// </summary>
        private static string errMessage = "Error executing server task";

        /// <summary>
        /// Sample inner exceptions to use.
        /// </summary>
        private static Exception[] inners = new Exception[]
            {
                new Exception("First"),
                new Exception("Second")
            };

        /// <summary>
        /// Tests constructor accuracy and its InnerExceptions property getter.
        /// </summary>
        [Test]
        public void TestCtor()
        {
            // instantiate with a sample error message and inner exceptions
            ServerTaskException e = new ServerTaskException(errMessage, inners);

            // assert error message and inner exceptions are correct
            // first element should be passed to base ctor
            Assert.AreEqual(errMessage, e.Message, "The error message is incorrect");
            Assert.AreSame(e.InnerExceptions, inners, "InnerExceptions is not initialized correctly");
            Assert.AreSame(e.InnerException, inners[0], "First element should be passed to base");
        }

        /// <summary>
        /// Tests constructor with a null error message and null inner exceptions.
        /// Expecting a default error message in the exception and null inner exceptions.
        /// </summary>
        [Test]
        public void TestCtorNull()
        {
            // instantiate with null error message and inner exceptions
            ServerTaskException e = new ServerTaskException(null, null);

            // assert error message is initialized to default message and inner exceptions are null
            Assert.IsNotNull(e.Message, "The error message is the default message");
            Assert.IsNull(e.InnerExceptions, "InnerExceptions is null");
            Assert.IsNull(e.InnerException, "First element is null");
        }

        /// <summary>
        /// Tests constructor with an empty error message and an empty array of inner exceptions.
        /// Expecting an empty error message and no first inner exception.
        /// </summary>
        [Test]
        public void TestCtorEmpty()
        {
            // instantiate with an empty error message and empty array of inner exceptions
            ServerTaskException e = new ServerTaskException(" ", new Exception[0]);

            // assert error message and InnerExceptions are empty
            Assert.AreEqual(" ", e.Message, "The error message is empty");
            Assert.AreEqual(0, e.InnerExceptions.Length, "InnerExceptions is empty");
            Assert.IsNull(e.InnerException, "First element is null");
        }

        /// <summary>
        /// Tests constructor with inner exceptions containing null element.
        /// Expecting inner exception is initialized accordingly.
        /// </summary>
        [Test]
        public void TestCtorNullElement()
        {
            // instantiate with array containing null element
            ServerTaskException e = new ServerTaskException(errMessage, new Exception[2]);

            // assert error message and InnerExceptions are empty
            Assert.AreEqual(errMessage, e.Message, "The error message is empty");
            Assert.AreEqual(2, e.InnerExceptions.Length, "InnerExceptions is empty");
            Assert.IsNull(e.InnerException, "First element is null");
        }

        /// <summary>
        /// Tests <see cref="ServerTaskException"/> inherits from <see cref="BuildException"/>.
        /// </summary>
        [Test]
        public void TestInheritance()
        {
            try
            {
                throw new ServerTaskException(errMessage, inners);
            }
            catch (Exception e)
            {
                // ok only if it's a direct subclass of BuildException
                Assert.AreEqual(typeof(BuildException), e.GetType().BaseType,
                    "ServerTaskException is not a direct subclass of BuildException");
            }
        }
    }
}
